

/*
 * HeartBeatMain
 *
 * The driver for the application.  The application just waits
 * for the user to hit the button before starting the heart.
 *
 * This application is then another view - a view that just plays
 * a sound in response to the updates.  This was just done to provide a
 * second example of a view without needing another class.
 *
 * The original author of the applet for this example is Cristobal Baray cbaray@cs.indiana.edu
 * See http://www.cs.indiana.edu/~cbaray/
 * Modified by Mark Fienup to make it an application 
 */

   import java.awt.*;
   import javax.sound.sampled.*;
   import java.awt.event.*;
   import java.io.*;
   import java.net.URL;
   import java.net.MalformedURLException;

   import java.util.Observer;
   import java.util.Observable;

    public class HeartBeatMain extends Frame //was Applet
    implements Observer {
   
      private Clip clip;
    	
      private Button StartButton;
   
      /** the heartbeat that we'll be monitoring */
      private static HeartBeat theHeart;	
      private static HeartBeatMain heartMain;
   
       public HeartBeatMain() {
      
         setSize(200,100);
         StartButton = new Button("click to start heart beat app");
         StartButton.addActionListener(new StartButtonListener());
         add("Center", StartButton);
         getClip();
      
      } // end HeartBeatMain
   
   
       private void getClip() {
         try {
         // From file
            AudioInputStream stream = AudioSystem.getAudioInputStream(new File("gong.au"));
         
         // From URL
         //        stream = AudioSystem.getAudioInputStream(new URL("http://hostname/audiofile"));
         
         // At present, ALAW and ULAW encodings must be converted
         // to PCM_SIGNED before it can be played
            AudioFormat format = stream.getFormat();
            if (format.getEncoding() != AudioFormat.Encoding.PCM_SIGNED) {
               format = new AudioFormat(
                  AudioFormat.Encoding.PCM_SIGNED,
                  format.getSampleRate(),
                  format.getSampleSizeInBits()*2,
                  format.getChannels(),
                  format.getFrameSize()*2,
                  format.getFrameRate(),
                  true);        // big endian
               stream = AudioSystem.getAudioInputStream(format, stream);
            }
         
         // Create the clip
            DataLine.Info info = new DataLine.Info(
               Clip.class, stream.getFormat(), 
               ((int)stream.getFrameLength()*format.getFrameSize()));
            clip = (Clip) AudioSystem.getLine(info);
         
         // This method does not return until the audio file is completely loaded
            clip.open(stream);
         }
             catch (UnsupportedAudioFileException ex) {
               System.out.println("Error: Unsupported");
            
            } 
             catch (Exception e) {
               System.out.println("Error in Audio stuff");
            }
      
      } // end getClip
   
       class StartButtonListener implements ActionListener {
          public void actionPerformed(java.awt.event.ActionEvent event)         {
            Object object = event.getSource();
            if (object == StartButton) {
               theHeart = new HeartBeat();
            
               new HeartView(theHeart).setVisible(true);
               new HeartController(theHeart).setVisible(true);
            
               theHeart.addObserver(heartMain);
            }
         }
      }
   
   /**
    * The update method for this View.  I couldn't
    * find a good EKG sound, so pretend it is the Tell Tale Heart...
    * If the sound hasn't been loaded, load it.
    *
    *@param model The model that this object is watching.
    *@param data The updated data object.
    */
       public void update(Observable model, Object data) {
      
         if (clip == null) {
            getClip();
         } // end if
      	
         if (clip != null) {
            clip.stop();
            clip.flush();
            clip.setFramePosition(0);
            clip.start();		
         } // end if
      } // end update
   
   /**
    * Acts as a controller as well - to stop the heart from continuing
    * to beat after we leave the page.
    *
    */
       public void stop() {
         theHeart.stopHeart();	
      }
   
   
   /**
    * The driver for the program - is absorbed into the 
    * applet class to make things easier.  To create an MVC
    * application, generally one creates a Model, then attaches
    * Views and Controllers to it.  Any number of Views and
    * Controllers can be added...
    *
    */
   
       public static void main(String args[]) {
      
         heartMain = new HeartBeatMain();
         heartMain.setVisible(true);
      
      } // end main	
   
   } // end class HeartBeatMain
