/*
 * HeartController
 *
 * A controller for the heart beat model. Has two buttons for
 * the user to change the speed that which the heart beats, and 
 * a button to stop the heart from beating.
 *
 */
 
import java.util.Observable;
import java.awt.*;
import java.awt.event.*;

public class HeartController extends Frame {

	private Button removeExcitement;
	private Button addExcitement;
	private Button stopButton;
		/** How much to increase the excitement level */
	private static final long INCREASE_AMOUNT = 2;
	
	/** How much to decrase the excitement level */
	private static final long DECREASE_AMOUNT = -1;
	
	/** A reference to the model */
	private HeartBeat myHeart;
	
	
	public HeartController() {
		setLayout(new FlowLayout(FlowLayout.CENTER,15,5));
		setVisible(false);
		setSize(213,40);
		removeExcitement = new Button("bore");
		removeExcitement.setBounds(19,5,46,20);
		ButtonListener buttonListener = new ButtonListener();
		removeExcitement.addActionListener(buttonListener);
		add(removeExcitement);
		addExcitement = new Button("excite");
		addExcitement.setBounds(80,5,55,20);
		addExcitement.addActionListener(buttonListener);
		add(addExcitement);
		stopButton = new java.awt.Button();
		stopButton.setLabel("stop");
		stopButton.setBounds(150,5,44,20);
		stopButton.addActionListener(buttonListener);
		add(stopButton);
		setTitle("Heart Controller");
		setResizable(false);
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
	} // end HeartController constructor

	public HeartController(String title)
	{
		this();
		setTitle(title);
	} // end HeartController constructor

	/** 
	 * Constructor that takes a model.  Calls the default
	 * constructor and then stores a reference to the model
	 * that this object will control.
	 *
	 *@param model The model this controller will control.
	 */
	public HeartController(Observable model) {
		this();
		myHeart = (HeartBeat) model;
	}



    /**
     * Shows or hides the component depending on the boolean flag shouldBeVisible.
     * @param shouldBeVisible  if true, show the component; otherwise, hide the component.
     * @see java.awt.Component#isVisible
     */
    public void setVisible(boolean shouldBeVisible)
	{
		if(shouldBeVisible)
		{
			setLocation(50, 50);
		}
		super.setVisible(shouldBeVisible);
	}

	public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();
	    
		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;


	class SymWindow extends WindowAdapter {
		public void windowClosing(WindowEvent event) {
			Object object = event.getSource();
			if (object == HeartController.this)
				HeartController_WindowClosing(event);
		} // end windowClosing
	} // end class SymWindow
	
	void HeartController_WindowClosing(WindowEvent event)
	{
		setVisible(false);		 // hide the Frame
	} // end 

	class ButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent event) {
			Object object = event.getSource();
			if (object == removeExcitement)
				removeExcitement_ActionPerformed(event);
			else if (object == addExcitement)
				addExcitement_ActionPerformed(event);
			else if (object == stopButton)
				stopButton_ActionPerformed(event);
		} // end actionPerformed
	} // end class ButtonListener


/////////////////////////////////////////////////////////////////////
	 
	/**
	 * When the user hits the remove button, tell the model to 
	 * adjust its excitementLevel appropriately.
	 */
	void removeExcitement_ActionPerformed(java.awt.event.ActionEvent event)	{
		myHeart.adjustExcitementLevel(INCREASE_AMOUNT);
	} // end removeExcitement_ActionPerformed
	
	/**
	 * When the user hits the excite button, tell the model to 
	 * adjust its excitementLevel appropriately.
	 */
	void addExcitement_ActionPerformed(java.awt.event.ActionEvent event) {
		myHeart.adjustExcitementLevel(DECREASE_AMOUNT);
	}// end addExcitement_ActionPerformed

	/**
	 * When the user hits the stop button, tell the model to halt.
	 */
	void stopButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		myHeart.stopHeart();
	} // end stopButton_ActionPerformed
} // end class HeartController
